#!/usr/bin/python
#===============================================================================
# CERN (BE-CO-HT)
# Read the CONV-TTL-BLO board ID and current temperature from DS18B20 chip
#===============================================================================
# author: Theodor Stana (t.stana@cern.ch)
#         Maciej Suminski (maciej.suminski@cern.ch)
#
# date of creation:
#
# version: 1.1
#
# description:
#   The script implements some classes to communicate to the OpenCores One-Wire
#   master core implemented inside the CONV-TTL-BLO and uses these classes to
#   retrieve the board ID and the current temperature on-board the CONV-TTL-BLO.
#
#===============================================================================
# GNU LESSER GENERAL PUBLIC LICENSE
#===============================================================================
# This source file is free software; you can redistribute it and/or modify it
# under the terms of the GNU Lesser General Public License as published by the
# Free Software Foundation; either version 2.1 of the License, or (at your
# option) any later version. This source is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU Lesser General Public License for more details. You should have
# received a copy of the GNU Lesser General Public License along with this
# source; if not, download it from http://www.gnu.org/licenses/lgpl-2.1.html
#===============================================================================
# last changes:
#    2017-02-02    M. Suminski          Updated to work with gateware v4
#                                       (read regs storing UID & temperature)
#===============================================================================
#  TODO: -
#===============================================================================
# Import system modules
import sys
import time
import os

# Import common modules
sys.path.append("../ei2c")
from ei2c import *
import ei2cdefine
sys.path.append("..")
import convttl_regs as regs

if __name__ == "__main__":

    FAMILY_CODE = 0x28

    # Get the IP, user and password for the ELMA crate from either user input,
    # or ei2cdefine.py
    if (ei2cdefine.ENABLED):
        ip   = ei2cdefine.HNAME
        user = ei2cdefine.USER
        pwd  = ei2cdefine.PWD
    else:
        ip   = raw_input("ELMA crate IP or hostname : ")
        user = raw_input("ELMA crate user name      : ")
        pwd  = raw_input("ELMA crate password       : ")
    testelma = EI2C(ip, user, pwd)
    testelma.open()

    # Ask for slot number
    while 1:
        try:
            slot = raw_input("Slot no.: ")
            slot = int(slot)
            break
        except TypeError as e:
            print("Please input a decimal slot number.")
        except SlotError as e:
            print(e.strerror)
        except KeyboardInterrupt:
            sys.exit();
        except:
            print("Unexpected error: ", sys.exc_info()[0])

    # Print which board is detected in the selected slot
    bid = testelma.get_bid(slot)

    # Reading of unique ID
    unique_id = ((testelma.read(slot, regs.UIDHR) << 32)
                | testelma.read(slot, regs.UIDLR))

    if(unique_id == -1):
        msg = "ERROR: TempID IC20: Unable to read 1-wire thermometer"
        print(msg)
    else:
        print("Unique ID: %.12X" % (unique_id))

    # Reading of temperature
    temp = testelma.read(slot, regs.TEMPR) / 16.0
    print("Current temperature: %3.3f *C" % temp)

    # Cheking if received values are reasonable
    if temp < 10.0 or temp > 50.0:
        msg = "ERROR: TempID IC20: Temperature: %d out of range[10 .. 50*C]" % (temp)
        print(msg)

    if((unique_id & 0xFF) != FAMILY_CODE):
        family_code = unique_id & 0xFF
        msg = "ERROR: TempID IC20: Invalid family code: 0x%.8X" % (family_code)
        print(msg)

    testelma.close()
